@@ -32,10 +32,9 @@
 #include <pwd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include <openssl/engine.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <stdio.h>
 
 #include "openconnect-internal.h"
 
@@ -335,12 +334,8 @@ static int process_http_response(struct openconnect_info *vpninfo, int *result,
 		}
 	}
 
-	if (closeconn || vpninfo->no_http_keepalive) {
-		SSL_free(vpninfo->https_ssl);
-		vpninfo->https_ssl = NULL;
-		close(vpninfo->ssl_fd);
-		vpninfo->ssl_fd = -1;
-	}
+	if (closeconn || vpninfo->no_http_keepalive)
+		openconnect_close_https(vpninfo, 0);
 
 	if (body)
 		body[done] = 0;
@@ -355,11 +350,17 @@ static int fetch_config(struct openconnect_info *vpninfo, char *fu, char *bu,
 	char buf[MAX_BUF_LEN];
 	char *config_buf = NULL;
 	int result, buflen;
-	unsigned char local_sha1_bin[SHA_DIGEST_LENGTH];
-	char local_sha1_ascii[(SHA_DIGEST_LENGTH * 2)+1];
-	EVP_MD_CTX c;
+	unsigned char local_sha1_bin[SHA1_SIZE];
+	char local_sha1_ascii[(SHA1_SIZE * 2)+1];
 	int i;
 
+	if (openconnect_open_https(vpninfo)) {
+		vpn_progress(vpninfo, PRG_ERR,
+			     _("Failed to open HTTPS connection to %s\n"),
+			     vpninfo->hostname);
+		return -EINVAL;
+	}
+
 	sprintf(buf, "GET %s%s HTTP/1.1\r\n", fu, bu);
 	sprintf(buf + strlen(buf), "Host: %s\r\n", vpninfo->hostname);
 	sprintf(buf + strlen(buf),  "User-Agent: %s\r\n", vpninfo->useragent);
@@ -374,7 +375,11 @@ static int fetch_config(struct openconnect_info *vpninfo, char *fu, char *bu,
 	}
 	sprintf(buf + strlen(buf),  "X-Transcend-Version: 1\r\n\r\n");
 
-	SSL_write(vpninfo->https_ssl, buf, strlen(buf));
+	if (openconnect_SSL_write(vpninfo, buf, strlen(buf)) != strlen(buf)) {
+		vpn_progress(vpninfo, PRG_ERR,
+			     _("Failed to send GET request for new config\n"));
+		return -EIO;
+	}
 
 	buflen = process_http_response(vpninfo, &result, NULL, &config_buf);
 	if (buflen < 0) {
@@ -387,11 +392,9 @@ static int fetch_config(struct openconnect_info *vpninfo, char *fu, char *bu,
 		return -EINVAL;
 	}
 
-	EVP_MD_CTX_init(&c);
-	EVP_Digest(config_buf, buflen, local_sha1_bin, NULL, EVP_sha1(), NULL);
-	EVP_MD_CTX_cleanup(&c);
+	openconnect_sha1(local_sha1_bin, config_buf, buflen);
 
-	for (i = 0; i < SHA_DIGEST_LENGTH; i++)
+	for (i = 0; i < SHA1_SIZE; i++)
 		sprintf(&local_sha1_ascii[i*2], "%02x", local_sha1_bin[i]);
 
 	if (strcasecmp(server_sha1, local_sha1_ascii)) {
@@ -444,14 +447,12 @@ static int run_csd_script(struct openconnect_info *vpninfo, char *buf, int bufle
 	close(fd);
 
 	if (!fork()) {
-		X509 *scert = SSL_get_peer_certificate(vpninfo->https_ssl);
-		X509 *ccert = SSL_get_certificate(vpninfo->https_ssl);
-		char scertbuf[EVP_MAX_MD_SIZE * 2 + 1];
-		char ccertbuf[EVP_MAX_MD_SIZE * 2 + 1];
+		char scertbuf[MD5_SIZE * 2 + 1];
+		char ccertbuf[MD5_SIZE * 2 + 1];
 		char *csd_argv[32];
 		int i = 0;
 
-		if (vpninfo->uid_csd != getuid()) {
+		if (vpninfo->uid_csd_given && vpninfo->uid_csd != getuid()) {
 			struct passwd *pw;
 
 			if (setuid(vpninfo->uid_csd)) {
@@ -471,12 +472,12 @@ static int run_csd_script(struct openconnect_info *vpninfo, char *buf, int bufle
 				exit(1);
 			}
 		}
-		if (vpninfo->uid_csd == 0 && !vpninfo->csd_wrapper) {
+		if (getuid() == 0 && !vpninfo->csd_wrapper) {
 			fprintf(stderr, _("Warning: you are running insecure "
 					  "CSD code with root privileges\n"
 					  "\t Use command line option \"--csd-user\"\n"));
 		}
-		if (vpninfo->uid_csd_given == 2) {	       
+		if (vpninfo->uid_csd_given == 2) {
 			/* The NM tool really needs not to get spurious output
 			   on stdout, which the CSD trojan spews. */
 			dup2(2, 1);
@@ -493,27 +494,17 @@ static int run_csd_script(struct openconnect_info *vpninfo, char *buf, int bufle
 		if (asprintf(&csd_argv[i++], "\"%s\"", vpninfo->authgroup?:"") == -1)
 			return -ENOMEM;
 
-		get_cert_md5_fingerprint(vpninfo, scert, scertbuf);
-		if (ccert)
-			get_cert_md5_fingerprint(vpninfo, ccert, ccertbuf);
-		else
-			ccertbuf[0] = 0;
-
+		openconnect_local_cert_md5(vpninfo, ccertbuf);
+		scertbuf[0] = 0;
+		get_cert_md5_fingerprint(vpninfo, vpninfo->peer_cert, scertbuf);
 		csd_argv[i++]= (char *)"-certhash";
 		if (asprintf(&csd_argv[i++], "\"%s:%s\"", scertbuf, ccertbuf) == -1)
 			return -ENOMEM;
+
 		csd_argv[i++]= (char *)"-url";
 		if (asprintf(&csd_argv[i++], "\"https://%s%s\"", vpninfo->hostname, vpninfo->csd_starturl) == -1)
 			return -ENOMEM;
-		/* WTF would it want to know this for? */
-		csd_argv[i++]= (char *)"-vpnclient";
-		csd_argv[i++]= (char *)"\"/opt/cisco/vpn/bin/vpnui";
-		csd_argv[i++]= (char *)"-connect";
-		if (asprintf(&csd_argv[i++], "https://%s/%s", vpninfo->hostname, vpninfo->csd_preurl) == -1)
-			return -ENOMEM;
-		csd_argv[i++]= (char *)"-connectparam";
-		if (asprintf(&csd_argv[i++], "#csdtoken=%s\"", vpninfo->csd_token) == -1)
-			return -ENOMEM;
+
 		csd_argv[i++]= (char *)"-langselen";
 		csd_argv[i++] = NULL;
 
@@ -642,7 +633,7 @@ int openconnect_obtain_cookie(struct openconnect_info *vpninfo)
 		free(form_buf);
 		form_buf = NULL;
 	}
-	if (!vpninfo->https_ssl && openconnect_open_https(vpninfo)) {
+	if (openconnect_open_https(vpninfo)) {
 		vpn_progress(vpninfo, PRG_ERR,
 			     _("Failed to open HTTPS connection to %s\n"),
 			     vpninfo->hostname);
@@ -693,12 +684,6 @@ int openconnect_obtain_cookie(struct openconnect_info *vpninfo)
 	if (result < 0)
 		return result;
 
-	/* Remember the peer's SSL certificate; it may disconnect during
-	   the response and then we wouldn't be able to find it */
-	if (vpninfo->peer_cert)
-		X509_free(vpninfo->peer_cert);
-	vpninfo->peer_cert = SSL_get_peer_certificate(vpninfo->https_ssl);
-
 	buflen = process_http_response(vpninfo, &result, NULL, &form_buf);
 	if (buflen < 0) {
 		/* We'll already have complained about whatever offended us */
@@ -735,12 +720,7 @@ int openconnect_obtain_cookie(struct openconnect_info *vpninfo)
 				/* Kill the existing connection, and a new one will happen */
 				free(vpninfo->peer_addr);
 				vpninfo->peer_addr = NULL;
-				if (vpninfo->https_ssl) {
-					SSL_free(vpninfo->https_ssl);
-					vpninfo->https_ssl = NULL;
-					close(vpninfo->ssl_fd);
-					vpninfo->ssl_fd = -1;
-				}
+				openconnect_close_https(vpninfo, 0);
 
 				for (opt = vpninfo->cookies; opt; opt = next) {
 					next = opt->next;
@@ -864,7 +844,7 @@ int openconnect_obtain_cookie(struct openconnect_info *vpninfo)
 					fu = tok + 3;
 				else if (!strncmp(tok, "fh:", 3)) {
 					if (!strncasecmp(tok+3, vpninfo->xmlsha1,
-							 SHA_DIGEST_LENGTH * 2))
+							 SHA1_SIZE * 2))
 						break;
 					sha = tok + 3;
 				}
