@@ -24,10 +24,11 @@
 
 #include <string.h>
 #include <errno.h>
+#include <stdlib.h>
 
 #include "openconnect-internal.h"
 
-struct openconnect_info *openconnect_vpninfo_new_with_cbdata (char *useragent,
+struct openconnect_info *openconnect_vpninfo_new (char *useragent,
 						  openconnect_validate_peer_cert_vfn validate_peer_cert,
 						  openconnect_write_new_config_vfn write_new_config,
 						  openconnect_process_auth_form_vfn process_auth_form,
@@ -36,7 +37,6 @@ struct openconnect_info *openconnect_vpninfo_new_with_cbdata (char *useragent,
 {
 	struct openconnect_info *vpninfo = calloc (sizeof(*vpninfo), 1);
 
-	vpninfo->mtu = 1406;
 	vpninfo->ssl_fd = -1;
 	vpninfo->cert_expire_warning = 60 * 86400;
 	vpninfo->useragent = openconnect_create_useragent (useragent);
@@ -46,6 +46,11 @@ struct openconnect_info *openconnect_vpninfo_new_with_cbdata (char *useragent,
 	vpninfo->progress = progress;
 	vpninfo->cbdata = privdata?:vpninfo;
 	vpninfo->cancel_fd = -1;
+#ifdef __APPLE__
+	vpninfo->csd_xmltag = "csdMac";
+#else
+	vpninfo->csd_xmltag = "csdLinux";
+#endif
 
 #ifdef ENABLE_NLS
 	bindtextdomain("openconnect", LOCALEDIR);
@@ -54,19 +59,6 @@ struct openconnect_info *openconnect_vpninfo_new_with_cbdata (char *useragent,
 	return vpninfo;
 }
 
-struct openconnect_info *openconnect_vpninfo_new (char *useragent,
-						  openconnect_validate_peer_cert_fn validate_peer_cert,
-						  openconnect_write_new_config_fn write_new_config,
-						  openconnect_process_auth_form_fn process_auth_form,
-						  openconnect_progress_fn progress)
-{
-	return openconnect_vpninfo_new_with_cbdata (useragent,
-						    (void *)validate_peer_cert,
-						    (void *)write_new_config,
-						    (void *)process_auth_form,
-						    (void *)progress, NULL);
-}
-
 static void free_optlist (struct vpn_option *opt)
 {
 	struct vpn_option *next;
@@ -81,7 +73,8 @@ static void free_optlist (struct vpn_option *opt)
 
 void openconnect_vpninfo_free (struct openconnect_info *vpninfo)
 {
-	openconnect_reset_ssl(vpninfo);
+	openconnect_close_https(vpninfo, 1);
+	free(vpninfo->peer_addr);
 	free_optlist(vpninfo->cookies);
 	free_optlist(vpninfo->cstp_options);
 	free_optlist(vpninfo->dtls_options);
@@ -90,7 +83,10 @@ void openconnect_vpninfo_free (struct openconnect_info *vpninfo)
 	free(vpninfo->redirect_url);
 	free(vpninfo->proxy_type);
 	free(vpninfo->proxy);
-	free(vpninfo->csd_scriptname);
+	if (vpninfo->csd_scriptname) {
+		unlink(vpninfo->csd_scriptname);
+		free(vpninfo->csd_scriptname);
+	}
 	free(vpninfo->csd_stuburl);
 	/* These are const in openconnect itself, but for consistency of
 	   the library API we do take ownership of the strings we're given,
@@ -99,8 +95,15 @@ void openconnect_vpninfo_free (struct openconnect_info *vpninfo)
 	if (vpninfo->cert != vpninfo->sslkey)
 		free((void *)vpninfo->sslkey);
 	free((void *)vpninfo->cert);
-	if (vpninfo->peer_cert)
+	if (vpninfo->peer_cert) {
+#if defined (OPENCONNECT_OPENSSL)
 		X509_free(vpninfo->peer_cert);
+#elif defined (OPENCONNECT_GNUTLS)
+		gnutls_x509_crt_deinit(vpninfo->peer_cert);
+#endif
+		vpninfo->peer_cert = NULL;
+	}
+	free(vpninfo->useragent);
 	/* No need to free deflate streams; they weren't initialised */
 	free(vpninfo);
 }
@@ -125,7 +128,7 @@ void openconnect_set_urlpath (struct openconnect_info *vpninfo, char *urlpath)
 	vpninfo->urlpath = urlpath;
 }
 
-void openconnect_set_xmlsha1 (struct openconnect_info *vpninfo, char *xmlsha1, int size)
+void openconnect_set_xmlsha1 (struct openconnect_info *vpninfo, const char *xmlsha1, int size)
 {
 	if (size != sizeof (vpninfo->xmlsha1))
 		return;
@@ -154,7 +157,7 @@ void openconnect_set_client_cert (struct openconnect_info *vpninfo, char *cert,
 		vpninfo->sslkey = cert;
 }
 
-struct x509_st *openconnect_get_peer_cert (struct openconnect_info *vpninfo)
+OPENCONNECT_X509 *openconnect_get_peer_cert (struct openconnect_info *vpninfo)
 {
 	return vpninfo->peer_cert;
 }
@@ -177,17 +180,11 @@ void openconnect_clear_cookie (struct openconnect_info *vpninfo)
 
 void openconnect_reset_ssl (struct openconnect_info *vpninfo)
 {
-	if (vpninfo->https_ssl) {
-		openconnect_close_https(vpninfo);
-	}
+	openconnect_close_https(vpninfo, 0);
 	if (vpninfo->peer_addr) {
 		free(vpninfo->peer_addr);
 		vpninfo->peer_addr = NULL;
 	}
-	if (vpninfo->https_ctx) {
-		SSL_CTX_free(vpninfo->https_ctx);
-		vpninfo->https_ctx = NULL;
-	}
 }
 
 int openconnect_parse_url (struct openconnect_info *vpninfo, char *url)
@@ -238,3 +235,33 @@ const char *openconnect_get_version (void)
 {
 	return openconnect_version_str;
 }
+
+int openconnect_has_pkcs11_support(void)
+{
+#if defined (OPENCONNECT_GNUTLS) && defined (HAVE_P11KIT)
+	return 1;
+#else
+	return 0;
+#endif
+}
+
+#if defined (OPENCONNECT_OPENSSL) && defined (HAVE_ENGINE)
+#include <openssl/engine.h>
+#endif
+int openconnect_has_tss_blob_support(void)
+{
+#if defined (OPENCONNECT_OPENSSL) && defined (HAVE_ENGINE)
+	ENGINE *e;
+
+	ENGINE_load_builtin_engines();
+
+	e = ENGINE_by_id("tpm");
+	if (e) {
+		ENGINE_free(e);
+		return 1;
+	}
+#elif defined (OPENCONNECT_GNUTLS) && defined (HAVE_TROUSERS)
+	return 1;
+#endif
+	return 0;
+}
